/*
   AngelCode Tool Box Library
   Copyright (c) 2004-2007 Andreas Jnsson
  
   This software is provided 'as-is', without any express or implied 
   warranty. In no event will the authors be held liable for any 
   damages arising from the use of this software.

   Permission is granted to anyone to use this software for any 
   purpose, including commercial applications, and to alter it and 
   redistribute it freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you 
      must not claim that you wrote the original software. If you use
      this software in a product, an acknowledgment in the product 
      documentation would be appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and 
      must not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source 
      distribution.
  
   Andreas Jnsson
   andreas@angelcode.com
*/

#include "acwin_statusbar.h"
#include <assert.h>

namespace acWindow
{

CStatusBar::CStatusBar() : CWindow()
{
}

int CStatusBar::Create(CWindow *parent)
{
	// Initialize the common control
	INITCOMMONCONTROLSEX icc;
	icc.dwSize = sizeof(icc);
	icc.dwICC = ICC_BAR_CLASSES;
	InitCommonControlsEx(&icc);

	HWND parentWnd = parent ? parent->GetHandle() : 0;
	HWND statusBar = CreateStatusWindow(WS_CHILD|WS_VISIBLE|WS_CLIPSIBLINGS|WS_CLIPCHILDREN, "", parentWnd, 0);
	if( Subclass(statusBar) < 0 )
		return -1;

	SetStatusText("");

	return 0;
}

LRESULT CStatusBar::MsgProc(UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch( msg )
	{
	case WM_DESTROY:
		DefWndProc(msg, wParam, lParam);
		Detach();
		return 0;
	}

	return DefWndProc(msg, wParam, lParam);
}

void CStatusBar::SetStatusText(const char *text)
{
	SendMessage(hWnd, SB_SETTEXT, SBT_NOBORDERS, (LPARAM)text);
}

}
