#include "choosefont.h"
#include "resource.h"
#include "commctrl.h"
#include "unicode.h"

using namespace acWindow;

typedef DWORD (_stdcall *GetGlyphIndices_t)(HDC hdc, LPCTSTR lpstr, int c, LPWORD pgi, DWORD fl);
extern GetGlyphIndices_t fGetGlyphIndicesA;
extern GetGlyphIndices_t fGetGlyphIndicesW;

int CALLBACK ChooseFontCallback(
  ENUMLOGFONTEX *lpelfe,    // logical-font data
  NEWTEXTMETRICEX *lpntme,  // physical-font data
  DWORD FontType,           // type of font
  LPARAM lParam             // application-defined data
);

int CALLBACK ChooseFontCallback2(
  ENUMLOGFONTEX *lpelfe,    // logical-font data
  NEWTEXTMETRICEX *lpntme,  // physical-font data
  DWORD FontType,           // type of font
  LPARAM lParam             // application-defined data
);

CChooseFont::CChooseFont() : CDialog()
{

}

int CChooseFont::DoModal(CWindow *parent)
{
	return CDialog::DoModal(MAKEINTRESOURCE(IDD_CHOOSEFONT), parent);
}

LRESULT CChooseFont::MsgProc(UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch( msg )
	{
	case WM_INITDIALOG:
		OnInit();
		return TRUE;

	case WM_COMMAND:
		switch( LOWORD(wParam) )
		{
		case IDOK:
			GetOptions();
			EndDialog(hWnd, IDOK);
			break;

		case IDCANCEL:
			EndDialog(hWnd, IDCANCEL);
			break;

		case IDC_FONT:
			if( HIWORD(wParam) == CBN_SELCHANGE )
				OnFontChange();
			break;

		case IDC_TEXTURE_FMT:
			if( HIWORD(wParam) == CBN_SELCHANGE )
				OnTextureChange();
			break;

		case IDC_BIT8:
		case IDC_BIT32:
			if( IsDlgButtonChecked(hWnd, IDC_BIT8) )
            	EnableWindow(GetDlgItem(hWnd, IDC_4CHNLPACK), FALSE);
			else
				EnableWindow(GetDlgItem(hWnd, IDC_4CHNLPACK), TRUE);
			break;

		case IDC_USEUNICODE:
		case IDC_USEOEM:
			EnableWindow(GetDlgItem(hWnd, IDC_CHARSET), !IsDlgButtonChecked(hWnd, IDC_USEUNICODE));
			break;	

		case IDC_OUTLINETHICKNESS:
			{
				int t = GetDlgItemInt(hWnd, IDC_OUTLINETHICKNESS, 0, FALSE);
				EnableWindow(GetDlgItem(hWnd, IDC_OUTLINE1CHNL), t > 0 ? TRUE : FALSE);
			}
			break;
		}
		break;
	}

	return DefWndProc(msg, wParam, lParam);
}

void CChooseFont::OnFontChange()
{
	char buf[256];
	GetDlgItemText(hWnd, IDC_CHARSET, buf, 256);
	if( strcmp(buf, "") != 0 )
		charSet = GetCharSet(buf);

	SendDlgItemMessage(hWnd, IDC_CHARSET, CB_RESETCONTENT, 0, 0);

	int idx = SendDlgItemMessage(hWnd, IDC_FONT, CB_GETCURSEL, 0, 0);
	if( idx != CB_ERR )
	{
		// Enumerate the charsets for the font
		LOGFONT lf;
		lf.lfCharSet = DEFAULT_CHARSET;
		lf.lfPitchAndFamily = 0;

		char buf[256];
		SendDlgItemMessage(hWnd, IDC_FONT, CB_GETLBTEXT, idx, (LPARAM)buf);

		strncpy(lf.lfFaceName, buf, LF_FACESIZE-1);
		lf.lfFaceName[LF_FACESIZE-1] = 0;

		HDC dc = GetDC(0);
		EnumFontFamiliesEx(dc, &lf, (FONTENUMPROC)ChooseFontCallback2, (LPARAM)this, 0);
		ReleaseDC(0, dc);
	}

	string str = GetCharSetName(charSet);
	int r = SendDlgItemMessage(hWnd, IDC_CHARSET, CB_SELECTSTRING, -1, (LPARAM)str.c_str());
	if( r == CB_ERR )
		SendDlgItemMessage(hWnd, IDC_CHARSET, CB_SETCURSEL, 0, 0);
}

void CChooseFont::OnInit()
{
	HDC dc;
	string str;

	// Get the device context for the display
	dc = GetDC(0);

	// Enumerate all fonts and types
	LOGFONT lf;
	lf.lfCharSet = DEFAULT_CHARSET; 
	lf.lfFaceName[0] = '\0';
	lf.lfPitchAndFamily = 0;

	EnumFontFamiliesEx(dc, &lf, (FONTENUMPROC)ChooseFontCallback, (LPARAM)this, 0);
	SendDlgItemMessage(hWnd, IDC_FONT, CB_SELECTSTRING, -1, (LPARAM)font.c_str());

	OnFontChange();

	str = GetCharSetName(charSet);
	SendDlgItemMessage(hWnd, IDC_CHARSET, CB_SELECTSTRING, -1, (LPARAM)str.c_str());
	SetDlgItemInt(hWnd, IDC_FONTSIZE, fontSize, FALSE);

	SendDlgItemMessage(hWnd, IDC_SIZESPIN, UDM_SETRANGE, 0, (LPARAM)MAKELONG(255, 1));
	SendDlgItemMessage(hWnd, IDC_SPINSAMPLING, UDM_SETRANGE, 0, (LPARAM)MAKELONG(4, 1)); 
	SendDlgItemMessage(hWnd, IDC_SCALEH_SPIN, UDM_SETRANGE, 0, (LPARAM)MAKELONG(200, 50)); 

	CheckDlgButton(hWnd, IDC_BOLD, isBold ? BST_CHECKED : BST_UNCHECKED);
	CheckDlgButton(hWnd, IDC_ITALIC, isItalic ? BST_CHECKED : BST_UNCHECKED);

	CheckDlgButton(hWnd, IDC_SMOOTH, useSmoothing ? BST_CHECKED : BST_UNCHECKED);

	if( fGetGlyphIndicesA )
		CheckDlgButton(hWnd, IDC_DISABLEBOXCHARS, disableBoxChars ? BST_CHECKED : BST_UNCHECKED);
	else
		EnableWindow(GetDlgItem(hWnd, IDC_DISABLEBOXCHARS), FALSE);

	CheckDlgButton(hWnd, IDC_USEUNICODE, useUnicode ? BST_CHECKED : BST_UNCHECKED);
	CheckDlgButton(hWnd, IDC_USEOEM, !useUnicode ? BST_CHECKED : BST_UNCHECKED);
	EnableWindow(GetDlgItem(hWnd, IDC_CHARSET), !IsDlgButtonChecked(hWnd, IDC_USEUNICODE));

	SetDlgItemInt(hWnd, IDC_ANTIALIASING, antiAliasing, FALSE);

	SetDlgItemInt(hWnd, IDC_SPACE_UP, paddingUp, FALSE);
	SetDlgItemInt(hWnd, IDC_SPACE_DOWN, paddingDown, FALSE);
	SetDlgItemInt(hWnd, IDC_SPACE_RIGHT, paddingRight, FALSE);
	SetDlgItemInt(hWnd, IDC_SPACE_LEFT, paddingLeft, FALSE);

	SetDlgItemInt(hWnd, IDC_SPACE_HORIZ, spacingHoriz, FALSE);
	SetDlgItemInt(hWnd, IDC_SPACE_VERT, spacingVert, FALSE);

	SetDlgItemInt(hWnd, IDC_SCALEH, scaleH, FALSE);

	ReleaseDC(0, dc);

	// Output texture
	SetDlgItemInt(hWnd, IDC_WIDTH, width, FALSE);
	SetDlgItemInt(hWnd, IDC_HEIGHT, height, FALSE);

	if( bitDepth == 32 )
	{
		CheckDlgButton(hWnd, IDC_BIT32, BST_CHECKED);
		CheckDlgButton(hWnd, IDC_BIT8, BST_UNCHECKED);

		CheckDlgButton(hWnd, IDC_4CHNLPACK, fourChnlPacked ? BST_CHECKED : BST_UNCHECKED);
	}
	else
	{
		CheckDlgButton(hWnd, IDC_BIT8, BST_CHECKED);
		CheckDlgButton(hWnd, IDC_BIT32, BST_UNCHECKED);

		CheckDlgButton(hWnd, IDC_4CHNLPACK, fourChnlPacked ? BST_CHECKED : BST_UNCHECKED);
        EnableWindow(GetDlgItem(hWnd, IDC_4CHNLPACK), FALSE);
	}

	CheckDlgButton(hWnd, IDC_DESC_TEXT, fontDescFormat == 0 ? BST_CHECKED : BST_UNCHECKED);
	CheckDlgButton(hWnd, IDC_DESC_XML,  fontDescFormat == 1 ? BST_CHECKED : BST_UNCHECKED);
	CheckDlgButton(hWnd, IDC_DESC_BIN,  fontDescFormat == 2 ? BST_CHECKED : BST_UNCHECKED);

	// Fill in the texture file format combo
	SendDlgItemMessage(hWnd, IDC_TEXTURE_FMT, CB_ADDSTRING, 0, (LPARAM)"dds - DirectDraw Surface");
	SendDlgItemMessage(hWnd, IDC_TEXTURE_FMT, CB_ADDSTRING, 0, (LPARAM)"png - Portable Network Graphics");
	SendDlgItemMessage(hWnd, IDC_TEXTURE_FMT, CB_ADDSTRING, 0, (LPARAM)"tga - Targa");
	SendDlgItemMessage(hWnd, IDC_TEXTURE_FMT, CB_SELECTSTRING, -1, (LPARAM)textureFormat.c_str());
	OnTextureChange();

	// Outline options
	SetDlgItemInt(hWnd, IDC_OUTLINETHICKNESS, outlineThickness, FALSE);
	CheckDlgButton(hWnd, IDC_OUTLINE1CHNL, is1ChnlEncoded ? BST_CHECKED : BST_UNCHECKED);
	if( outlineThickness == 0 )
		EnableWindow(GetDlgItem(hWnd, IDC_OUTLINE1CHNL), FALSE);

	SendDlgItemMessage(hWnd, IDC_SPINTHICKNESS, UDM_SETRANGE, 0, (LPARAM)MAKELONG(32, 0)); 
}

void CChooseFont::OnTextureChange()
{
	// Clear the compression combo
	SendDlgItemMessage(hWnd, IDC_TEXTURE_COMPRESSION, CB_RESETCONTENT, 0, 0);

	// Which texture format is chosen?
	int idx = SendDlgItemMessage(hWnd, IDC_TEXTURE_FMT, CB_GETCURSEL, 0, 0);
	if( idx != CB_ERR )
	{
		char buf[256];
		SendDlgItemMessage(hWnd, IDC_TEXTURE_FMT, CB_GETLBTEXT, idx, (LPARAM)buf);

		textureFormat.assign(buf, 3);
	}

	// Add the compression options
	if( textureFormat == "tga" )
	{
		SendDlgItemMessage(hWnd, IDC_TEXTURE_COMPRESSION, CB_ADDSTRING, 0, (LPARAM)"None");
//		SendDlgItemMessage(hWnd, IDC_TEXTURE_COMPRESSION, CB_ADDSTRING, 0, (LPARAM)"Run-length encoded");
	}
	else if( textureFormat == "png" )
	{
		SendDlgItemMessage(hWnd, IDC_TEXTURE_COMPRESSION, CB_ADDSTRING, 0, (LPARAM)"Deflate");
	}
	else if( textureFormat == "dds" )
	{
		SendDlgItemMessage(hWnd, IDC_TEXTURE_COMPRESSION, CB_ADDSTRING, 0, (LPARAM)"None");
//		SendDlgItemMessage(hWnd, IDC_TEXTURE_COMPRESSION, CB_ADDSTRING, 0, (LPARAM)"DXT1");
//		SendDlgItemMessage(hWnd, IDC_TEXTURE_COMPRESSION, CB_ADDSTRING, 0, (LPARAM)"DXT3");
//		SendDlgItemMessage(hWnd, IDC_TEXTURE_COMPRESSION, CB_ADDSTRING, 0, (LPARAM)"DXT5");
	}

	SendDlgItemMessage(hWnd, IDC_TEXTURE_COMPRESSION, CB_SETCURSEL, 0, 0);
}

int CALLBACK ChooseFontCallback(
  ENUMLOGFONTEX *lpelfe,    // logical-font data
  NEWTEXTMETRICEX *lpntme,  // physical-font data
  DWORD FontType,           // type of font
  LPARAM lParam             // application-defined data
)
{
	CDialog *dlg = (CChooseFont *)lParam;

	// Add font name to combobox
	int idx = SendDlgItemMessage(dlg->GetHandle(), IDC_FONT, CB_FINDSTRINGEXACT, 0, (LPARAM)(char*)lpelfe->elfLogFont.lfFaceName);
	if( idx == CB_ERR )
		SendDlgItemMessage(dlg->GetHandle(), IDC_FONT, CB_ADDSTRING, 0, (LPARAM)(char*)lpelfe->elfLogFont.lfFaceName);

	return 1;
}

int CALLBACK ChooseFontCallback2(
  ENUMLOGFONTEX *lpelfe,
  NEWTEXTMETRICEX *lpntme,
  DWORD FontType,
  LPARAM lParam
)
{
	CChooseFont *dlg = (CChooseFont *)lParam;

	string str = GetCharSetName(lpelfe->elfLogFont.lfCharSet);

	// Add charset to combobox
	int idx = SendDlgItemMessage(dlg->GetHandle(), IDC_CHARSET, CB_FINDSTRINGEXACT, 0, (LPARAM)str.c_str());
	if( idx == CB_ERR )
		SendDlgItemMessage(dlg->GetHandle(), IDC_CHARSET, CB_ADDSTRING, 0, (LPARAM)str.c_str());

	return 1;
}

void CChooseFont::GetOptions()
{
	char buf[256];

	GetDlgItemText(hWnd, IDC_FONT, buf, 256);
	font = buf;

	GetDlgItemText(hWnd, IDC_CHARSET, buf, 256);
	charSet = GetCharSet(buf);

	fontSize = GetDlgItemInt(hWnd, IDC_FONTSIZE, 0, FALSE);

	antiAliasing = GetDlgItemInt(hWnd, IDC_ANTIALIASING, 0, FALSE);
	if( antiAliasing < 1 ) antiAliasing = 1;
	if( antiAliasing > 4 ) antiAliasing = 4;

	paddingUp = GetDlgItemInt(hWnd, IDC_SPACE_UP, 0, FALSE);
	paddingDown = GetDlgItemInt(hWnd, IDC_SPACE_DOWN, 0, FALSE);
	paddingRight = GetDlgItemInt(hWnd, IDC_SPACE_RIGHT, 0, FALSE);
	paddingLeft = GetDlgItemInt(hWnd, IDC_SPACE_LEFT, 0, FALSE);

	spacingHoriz = GetDlgItemInt(hWnd, IDC_SPACE_HORIZ, 0, FALSE);
	spacingVert = GetDlgItemInt(hWnd, IDC_SPACE_VERT, 0, FALSE);

	scaleH = GetDlgItemInt(hWnd, IDC_SCALEH, 0, FALSE);
	if( scaleH < 1 ) scaleH = 1;

	isBold = IsDlgButtonChecked(hWnd, IDC_BOLD) == BST_CHECKED;
	isItalic = IsDlgButtonChecked(hWnd, IDC_ITALIC) == BST_CHECKED;

	useSmoothing = IsDlgButtonChecked(hWnd, IDC_SMOOTH) == BST_CHECKED;
	
	disableBoxChars = IsDlgButtonChecked(hWnd, IDC_DISABLEBOXCHARS) == BST_CHECKED;

	// Output texture
	width = GetDlgItemInt(hWnd, IDC_WIDTH, 0, FALSE);
	height = GetDlgItemInt(hWnd, IDC_HEIGHT, 0, FALSE);

	if( IsDlgButtonChecked(hWnd, IDC_BIT8) ) 
		bitDepth = 8; 
	else 
		bitDepth = 32;
	fourChnlPacked = IsDlgButtonChecked(hWnd, IDC_4CHNLPACK) ? true : false;

	if( IsDlgButtonChecked(hWnd, IDC_DESC_TEXT) ) fontDescFormat = 0;
	if( IsDlgButtonChecked(hWnd, IDC_DESC_XML)  ) fontDescFormat = 1;
	if( IsDlgButtonChecked(hWnd, IDC_DESC_BIN)  ) fontDescFormat = 2;

	if( IsDlgButtonChecked(hWnd, IDC_USEUNICODE) ) useUnicode = true; else useUnicode = false;

	GetDlgItemText(hWnd, IDC_TEXTURE_FMT, buf, 256);
	textureFormat.assign(buf, 3);

	textureCompression = SendDlgItemMessage(hWnd, IDC_TEXTURE_COMPRESSION, CB_GETCURSEL, 0, 0);

	outlineThickness = GetDlgItemInt(hWnd, IDC_OUTLINETHICKNESS, 0, FALSE);
	is1ChnlEncoded = IsDlgButtonChecked(hWnd, IDC_OUTLINE1CHNL) ? true : false;
}






