#include <math.h>
#include "fontchar.h"

CFontChar::CFontChar()
{
	charImg = 0;
}

CFontChar::~CFontChar()
{
	if( charImg )
		delete charImg;
}

void CFontChar::CreateFromImage(int ch, cImage *image)
{
	isChar  = false;
	id      = ch;
	width   = image->width;
	height  = image->height;
	advance = width;
	xoffset = 0;
	yoffset = 0;
	colored = true;

	charImg = new cImage(image->width, image->height);
	charImg->isTopDown = true;

	// Copy the input image to charImg
	memcpy(charImg->pixels, image->pixels, width*height*4);
}

bool CFontChar::FitsInOneChannel(bool is1ChnlEncoded)
{
	if( !isChar ) return false;
	if( !colored ) return true;

	return is1ChnlEncoded;
}

bool CFontChar::HasOutline()
{
	return isChar && colored;
}

BYTE CFontChar::GetPixelValue(int x, int y)
{
	if( isChar )
	{
		if( colored )
		{
			DWORD color = charImg->pixels[y*charImg->width+x];
			if( BYTE(color) )
			{
				return 0x80 | (((BYTE)(color))>>1);
			}
			else
			{
				return BYTE(color>>25);
			}
		}
		else
		{
			return (BYTE)charImg->pixels[y*charImg->width+x];
		}
	}

	return 0;
}

void CFontChar::DrawChar(HFONT font, int ch, int scaleH, int aa, bool useUnicode)
{
	colored = false;
	id      = ch;
	isChar  = true;

	// Create a memory dc
	HDC dc = CreateCompatibleDC(0);

	// Scale the coordinate system so that the font is stretched
	if( SetGraphicsMode(dc, GM_ADVANCED) )
	{
		XFORM mtx;
		mtx.eM11 = 1.0f;
		mtx.eM12 = 0;
		mtx.eM21 = 0;
		mtx.eM22 = float(scaleH)/100.0f;
		mtx.eDx = 0;
		mtx.eDy = 0;
		SetWorldTransform(dc, &mtx);
	}

	HFONT oldFont = (HFONT)SelectObject(dc, font);

	// Determine the size needed for the char
	ABC abc;
	if( useUnicode )
	{
		if( !GetCharABCWidthsW(dc, ch, ch, &abc) )
		{
			// Use GetCharWidth32() instead
			GetCharWidth32W(dc, ch, ch, &width);

			abc.abcA = abc.abcC = 0;
			abc.abcB = (unsigned)width;
		}
	}
	else
	{
		if( !GetCharABCWidths(dc, ch, ch, &abc) )
		{
			// Use GetCharWidth32() instead
			GetCharWidth32(dc, ch, ch, &width);

			abc.abcA = abc.abcC = 0;
			abc.abcB = (unsigned)width;
		}
	}

	int tmpWidth = abc.abcB;

	TEXTMETRIC tm;
	GetTextMetrics(dc, &tm);

	int tmpHeight = int(ceilf(tm.tmHeight*float(scaleH)/100.0f));

	// Create the image that will receive the pixels
	width = int(ceilf(float(tmpWidth)/aa));
	height = int(ceilf(float(tmpHeight)/aa));
	charImg = new cImage(width, height);
	charImg->isTopDown = true;
	memset(charImg->pixels, 0, charImg->width * charImg->height * 4);

	// Draw the character
	DWORD *pixels;
	BITMAPINFO bmi;
	ZeroMemory(&bmi, sizeof(BITMAPINFO));
    bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    bmi.bmiHeader.biWidth = charImg->width*aa;
    bmi.bmiHeader.biHeight = -charImg->height*aa;
    bmi.bmiHeader.biPlanes = 1;
    bmi.bmiHeader.biBitCount = 32;         
    bmi.bmiHeader.biCompression = BI_RGB;
    bmi.bmiHeader.biSizeImage = charImg->width*aa * charImg->height*aa * 4;

	HBITMAP bm = CreateDIBSection(dc, &bmi, DIB_RGB_COLORS, (void**)&pixels, 0, 0);
	HBITMAP oldBM = (HBITMAP)SelectObject(dc, bm);

	HBRUSH black = (HBRUSH)GetStockObject(BLACK_BRUSH);

	memset(pixels, 0, bmi.bmiHeader.biSizeImage);

	SetTextColor(dc, RGB(255,255,255));
	SetBkColor(dc, RGB(0,0,0));
	SetBkMode(dc, TRANSPARENT);

	if( useUnicode )
		TextOutW(dc, -abc.abcA, 0, (WCHAR*)&ch, 1);
	else
		TextOutA(dc, -abc.abcA, 0, (char*)&ch, 1);

	GdiFlush();

	// Retrieve the pixels to the image
	if( aa == 1 )
	{
		SelectObject(dc, oldBM);

		memcpy(charImg->pixels, pixels, charImg->width*charImg->height*4);
	}
	else if( aa == 2 )
	{
		int w = charImg->width*aa;
		for( int y = 0; y < charImg->height; y++ )
		{
			for( int x = 0; x < charImg->width; x++ )
			{
				int c = pixels[x*2 + y*2*w] & 0xFF;
				c += pixels[x*2+1 + y*2*w] & 0xFF;

				c += pixels[x*2 + (y*2+1)*w] & 0xFF;
				c += pixels[x*2+1 + (y*2+1)*w] & 0xFF;

				c /= 4;

				charImg->pixels[y*charImg->width + x] = c | (c<<8) | (c<<16) | (c<<24);
			}
		}
	}
	else if( aa == 3 )
	{
		int w = charImg->width*aa;
		for( int y = 0; y < charImg->height; y++ )
		{
			for( int x = 0; x < charImg->width; x++ )
			{
				int c = pixels[x*3 + y*3*w] & 0xFF;
				c += pixels[x*3+1 + y*3*w] & 0xFF;
				c += pixels[x*3+2 + y*3*w] & 0xFF;

				c += pixels[x*3+0 + (y*3+1)*w] & 0xFF;
				c += pixels[x*3+1 + (y*3+1)*w] & 0xFF;
				c += pixels[x*3+2 + (y*3+1)*w] & 0xFF;

				c += pixels[x*3+0 + (y*3+2)*w] & 0xFF;
				c += pixels[x*3+1 + (y*3+2)*w] & 0xFF;
				c += pixels[x*3+2 + (y*3+2)*w] & 0xFF;

				c /= 9;

				charImg->pixels[y*charImg->width + x] = c | (c<<8) | (c<<16) | (c<<24);
			}
		}
	}
	else if( aa == 4 )
	{
		int w = charImg->width*aa;
		for( int y = 0; y < charImg->height; y++ )
		{
			for( int x = 0; x < charImg->width; x++ )
			{
				int c = pixels[x*4 + y*4*w] & 0xFF;
				c += pixels[x*4+1 + y*4*w] & 0xFF;
				c += pixels[x*4+2 + y*4*w] & 0xFF;
				c += pixels[x*4+3 + y*4*w] & 0xFF;

				c += pixels[x*4+0 + (y*4+1)*w] & 0xFF;
				c += pixels[x*4+1 + (y*4+1)*w] & 0xFF;
				c += pixels[x*4+2 + (y*4+1)*w] & 0xFF;
				c += pixels[x*4+3 + (y*4+1)*w] & 0xFF;

				c += pixels[x*4+0 + (y*4+2)*w] & 0xFF;
				c += pixels[x*4+1 + (y*4+2)*w] & 0xFF;
				c += pixels[x*4+2 + (y*4+2)*w] & 0xFF;
				c += pixels[x*4+3 + (y*4+2)*w] & 0xFF;

				c += pixels[x*4+0 + (y*4+3)*w] & 0xFF;
				c += pixels[x*4+1 + (y*4+3)*w] & 0xFF;
				c += pixels[x*4+2 + (y*4+3)*w] & 0xFF;
				c += pixels[x*4+3 + (y*4+3)*w] & 0xFF;

				c /= 16;

				charImg->pixels[y*charImg->width + x] = c | (c<<8) | (c<<16) | (c<<24);
			}
		}
	}
	
	xoffset = abc.abcA/aa;
	advance = (abc.abcA + abc.abcB + abc.abcC)/aa;
	yoffset = 0;

	// Discount scanlines that are not drawn
	for( int y = 0; y < charImg->height; y++ )
	{
		bool empty = true;
		for( int x = 0; x < charImg->width; x++ )
		{
			if( charImg->pixels[y*charImg->width+x] != 0 )
			{
				empty = false;
				break;
			}
		}

		if( empty )
		{
			height--;
			yoffset++;
		}
		else
			break;
	}

	// Discount scanlines that are not drawn
	for( int y = charImg->height-1; y >= yoffset; y-- )
	{
		bool empty = true;
		for( int x = 0; x < charImg->width; x++ )
		{
			if( charImg->pixels[y*charImg->width+x] != 0 )
			{
				empty = false;
				break;
			}
		}

		if( empty )
		{
			height--;
		}
		else
			break;
	}

	// Remove the empty scanlines
	if( yoffset )
	{
		for( int y = 0; y < height; y++ )
			for( int x = 0; x < width; x++ )
				charImg->pixels[y*charImg->width+x] = charImg->pixels[(y+yoffset)*charImg->width+x];			
	}
	charImg->height = height;

	// Clean up
	SelectObject(dc, oldFont);
	SelectObject(dc, oldBM);

	DeleteObject(bm);

	DeleteDC(dc);
}

void CFontChar::AddOutline(int thickness)
{
	if( !charImg->height || !charImg->width )
		return;

	colored = true;

	width  += thickness*2;
	height += thickness*2;
	xoffset -= thickness;
	yoffset -= thickness;

	cImage img;
	img.Create(charImg->width+2*thickness, charImg->height+2*thickness);
	img.Clear(0);

	// Create the kernel
	int kernelWidth = thickness*2+1;
	float *kernel = new float[kernelWidth*kernelWidth];

	// Circular kernel with anti-aliasing
	for( int y = 0; y < kernelWidth; y++ )
	{
		for( int x = 0; x < kernelWidth; x++ )
		{
			float val;
			if( x == thickness || y == thickness )
				val = 1;
			else 
			{
				val = thickness+1 - thickness*float((x-thickness)*(x-thickness)+(y-thickness)*(y-thickness))/(thickness*thickness);
				if( val > 1 ) val = 1;
				else if( val < 0 ) val = 0;
			}
			kernel[y*kernelWidth+x] = val;
		}
	}

	// Create the outline
	for( int y1 = 0; y1 < charImg->height; y1++ )
	{
		for( int x1 = 0; x1 < charImg->width; x1++ )
		{
			DWORD cs = charImg->pixels[y1*charImg->width+x1] & 0xFF;
			for( int y2 = 0; y2 < kernelWidth; y2++ )
			{
				for( int x2 = 0; x2 < kernelWidth; x2++ )
				{
					if( x2 == thickness && y2 == thickness )
					{
						if( cs )
							img.pixels[(y1+y2)*img.width+(x1+x2)] = 0xFF000000|(cs<<16)|(cs<<8)|cs;
					}
					else
					{
						DWORD val = DWORD(cs*kernel[y2*kernelWidth+x2])<<24;
						DWORD cd = img.pixels[(y1+y2)*img.width+(x1+x2)];
						if( val > cd )
							img.pixels[(y1+y2)*img.width+(x1+x2)] = val;
					}
				}
			}
		}
	}

	delete[] kernel;

	// Move the new image to charImg
	charImg->width = img.width;
	charImg->height = img.height;
	delete[] charImg->pixels;
	charImg->pixels = img.pixels;
	img.pixels = 0;
}