#ifndef FONTGEN_H
#define FONTGEN_H

#include <windows.h>

#include <string>
using std::string;
#include <vector>
using std::vector;

#include "fontpage.h"

class CFontChar;

struct SIconImage
{
	SIconImage() {image = 0;}
	~SIconImage() {if( image ) delete image;}
	string  fileName;
	int     id;
	cImage *image;
};

class CFontGen
{
public:
	CFontGen();
	~CFontGen();

	// Generate font pages asynchronously
	int     GeneratePages();
	void    Abort();
	int     GetStatus();
	int     GetStatusCounter();

	// Icon images
	int     AddIconImage(const char *file, int id);
	int     GetIconImageCount();
	int     GetIconImageInfo(int n, string &filename, int &id);
	int     DeleteIconImage(int id);
	int     UpdateIconImage(int oldId, int id, const char *file);
	void    ClearIconImages();

	// Select individual characters
	bool    IsSelected(int charIdx);
	int     SetSelected(int charIdx, bool set);
	int     SelectSubset(int subset, bool set, bool entireRange);
	int     IsSubsetSelected(int subset);
	bool    IsDisabled(int charIdx);
	bool    DidNotFit(int charIdx);
	bool    IsBoxCharsDisabled();
	int     SetBoxCharsDisabled(bool set);
	int     GetNumCharsSelected();
	int     GetNumCharsAvailable();
	int     SelectCharsFromFile(const char *filename);

	// Failed characters
	int     GetNumFailedChars();
	int     FindNextFailedCharacterSubset(int startSubset);
	void    ClearFailedCharacters();
	int     SubsetFromChar(int ch);

	// Font properties
	string  GetFontName();           int SetFontName(string &name);
	int     GetCharSet();            int SetCharSet(int charSet);
	int     GetFontSize();           int SetFontSize(int fontSize);
	bool    IsBold();                int SetBold(bool set);
	bool    IsItalic();              int SetItalic(bool set);
	int     GetAntiAliasingLevel();  int SetAntiAliasingLevel(int level);
	bool    IsUsingSmoothing();      int SetUseSmoothing(bool set);
	bool    IsUsingUnicode();        int SetUseUnicode(bool set);

	// Character padding and spacing
	int     GetPaddingDown();        int SetPaddingDown(int pad);
	int     GetPaddingUp();          int SetPaddingUp(int pad);
	int     GetPaddingLeft();        int SetPaddingLeft(int pad);
	int     GetPaddingRight();       int SetPaddingRight(int pad);
	int     GetSpacingHoriz();       int SetSpacingHoriz(int space);
	int     GetSpacingVert();        int SetSpacingVert(int space);
	int     GetScaleHeight();        int SetScaleHeight(int scale);
									
	// Output font file				
	int     GetOutWidth();           int SetOutWidth(int width);
	int     GetOutHeight();          int SetOutHeight(int height);
	int     GetOutBitDepth();        int SetOutBitDepth(int bitDepth);
	int     GetFontDescFormat();     int SetFontDescFormat(int format);
	bool    Is4ChnlPacked();         int Set4ChnlPacked(bool set);
	string  GetTextureFormat();      int SetTextureFormat(string &format);
	int     GetTextureCompression(); int SetTextureCompression(int compression);

	// Outline
	int     GetOutlineThickness();   int SetOutlineThickness(int thickness);
	bool    Is1ChnlEncoded();        int Set1ChnlEncoded(bool set);

	// Call this after updating the font properties
	int     Prepare();

	// A helper function for creating the font object
	HFONT   CreateFont(int fontSize);

	// Visualize pages
	int     GetNumPages();
	cImage *GetPageImage(int page, int channel);

	// Save the font to disk
	int     SaveFont(const char *filename);

	// Configuration
	int     SaveConfiguration(const char *filename);
	int     LoadConfiguration(const char *filename);

protected:
	friend class CFontPage;

	void ResetFont();
	void ClearPages();
	int  CreatePage();
	void DetermineExistingChars();

	static void __cdecl GenerateThread(CFontGen *fontGen);
	void InternalGeneratePages();

	bool fontChanged;

	bool isWorking;
	bool stopWorking;
	int  status;
	int  counter;
	bool disableBoxChars;
	bool arePagesGenerated;

	// Font properties
	string fontName;
	int    charSet;
	int    fontSize;
	int    aa;
	int    scaleH;
	bool   useSmoothing;
	bool   isBold;
	bool   isItalic;
	bool   useUnicode;

	// Char alignment options
	int paddingDown;
	int paddingUp;
	int paddingRight;
	int paddingLeft;	
	int spacingHoriz;
	int spacingVert;

	// File output options
	int    outWidth;
	int    outHeight;
	int    outBitDepth;
	int    fontDescFormat;
	bool   fourChnlPacked;
	string textureFormat;
	int    textureCompression;

	// Outline
	int    outlineThickness;
	bool   is1ChnlEncoded;

	// Characters
	int  numCharsSelected;
	int  numCharsAvailable;
	bool disabled[65536];
	bool selected[65536];
	bool noFit[65536];
	CFontChar *chars[65536];

	// Font textures
	vector<CFontPage *> pages;

	// Icon images
	vector<SIconImage *> iconImages;
};

#endif