#include <windows.h>
#include <stdio.h>

#include "dynamic_funcs.h"
#include "ac_string_util.h"

#include "unicode.h"

UnicodeSubset_t UnicodeSubsets[] = {
{"Basic Latin + Supplement"                 , 0x0000  , 0x00FF, 0},
{"Latin Extended-A"                         , 0x0100  , 0x017F, 1},
{"Latin Extended-B"                         , 0x0180  , 0x024F, 2},
{"IPA Extensions"                           , 0x0250  , 0x02AF, 3},
{"Spacing Modifier Letters"                 , 0x02B0  , 0x02FF, 4},
{"Combining Diacritical Marks"              , 0x0300  , 0x036F, 5},
{"Greek and Coptic"                         , 0x0370  , 0x03FF, 6},
{"Cyrillic"                                 , 0x0400  , 0x04FF, 7},
{"Cyrillic Supplementary"                   , 0x0500  , 0x052F, 8},
{"Armenian"                                 , 0x0530  , 0x058F, 9},
{"Basic Hebrew"                             , 0x0590  , 0x05FF, 10},
{"Basic Arabic"                             , 0x0600  , 0x06FF, 11},
{"Syriac"                                   , 0x0700  , 0x074F, 12},
{"(0x0750 - 0x077F)"                        , 0x0750  , 0x077F, 13},
{"Thaana"                                   , 0x0780  , 0x07BF, 14},
{"(0x07C0 - 0x08FF)"                        , 0x07C0  , 0x08FF, 15},
{"Devanagari"                               , 0x0900  , 0x097F, 16},
{"Bengali"                                  , 0x0980  , 0x09FF, 17},
{"Gurmukhi"                                 , 0x0A00  , 0x0A7F, 18},
{"Gujarati"                                 , 0x0A80  , 0x0AFF, 19},
{"Oriya"                                    , 0x0B00  , 0x0B7F, 20},
{"Tamil"                                    , 0x0B80  , 0x0BFF, 21},
{"Telugu"                                   , 0x0C00  , 0x0C7F, 22},
{"Kannada"                                  , 0x0C80  , 0x0CFF, 23},
{"Malayalam"                                , 0x0D00  , 0x0D7F, 24},
{"Sinhala"                                  , 0x0D80  , 0x0DFF, 25},
{"Thai"                                     , 0x0E00  , 0x0E7F, 26},
{"Lao"                                      , 0x0E80  , 0x0EFF, 27},
{"Tibetan"                                  , 0x0F00  , 0x0FFF, 28},
{"Myanmar"                                  , 0x1000  , 0x109F, 29},
{"Georgian"                                 , 0x10A0  , 0x10FF, 30},
{"Hangul Jamo"                              , 0x1100  , 0x11FF, 31},
{"Ethiopic"                                 , 0x1200  , 0x12BF, 32},
{"(0x12C0 - 0x139F)"                        , 0x12C0  , 0x139F, 33},
{"Cherokee"                                 , 0x13A0  , 0x13FF, 34},
{"Canadian Aboriginal Syllabics (1/3)"      , 0x1400  , 0x14FF, 35},
{"Canadian Aboriginal Syllabics (2/3)"      , 0x1500  , 0x15FF, 35},
{"Canadian Aboriginal Syllabics (3/3)"      , 0x1600  , 0x167F, 35},
{"Ogham"                                    , 0x1680  , 0x169F, 36},
{"Runic"                                    , 0x16A0  , 0x16FF, 37},
{"Tagalog"                                  , 0x1700  , 0x171F, 38},
{"Hanunoo"                                  , 0x1720  , 0x173F, 39},
{"Buhid"                                    , 0x1740  , 0x175F, 40},
{"Tagbanwa"                                 , 0x1760  , 0x177F, 41},
{"Khmer"                                    , 0x1780  , 0x17FF, 42},
{"Mongolian"                                , 0x1800  , 0x18AF, 43},
{"(0x18B0 - 0x19DF)"                        , 0x18B0  , 0x19DF, 44},
{"Khmer Symbols"                            , 0x19E0  , 0x19FF, 45},
{"(0x1A00 - 0x1DFF)"                        , 0x1A00  , 0x1DFF, 46},
{"Latin Extended Additional"                , 0x1E00  , 0x1EFF, 47},
{"Greek Extended"                           , 0x1F00  , 0x1FFF, 48},
{"General Punctuation"                      , 0x2000  , 0x206F, 49},
{"Subscripts and Superscripts"              , 0x2070  , 0x209F, 50},
{"Currency Symbols"                         , 0x20A0  , 0x20CF, 51},
{"Combining Diacritical Marks for Symbols"  , 0x20D0  , 0x20FF, 52},
{"Letter-like Symbols"                      , 0x2100  , 0x214F, 53},
{"Number Forms"                             , 0x2150  , 0x218F, 54},
{"Arrows"                                   , 0x2190  , 0x21FF, 55},
{"Mathematical Operators"                   , 0x2200  , 0x22FF, 56},
{"Miscellaneous Technical"                  , 0x2300  , 0x23FF, 57},
{"Control Pictures"                         , 0x2400  , 0x243F, 58},
{"Optical Character Recognition"            , 0x2440  , 0x245F, 59},
{"Enclosed Alphanumerics"                   , 0x2460  , 0x24FF, 60},
{"Box Drawing"                              , 0x2500  , 0x257F, 61},
{"Block Elements"                           , 0x2580  , 0x259F, 62},
{"Geometric Shapes"                         , 0x25A0  , 0x25FF, 63},
{"Miscellaneous Symbols"                    , 0x2600  , 0x26FF, 64},
{"Dingbats"                                 , 0x2700  , 0x27BF, 65},
{"Miscellaneous Mathematical Symbols-A"     , 0x27C0  , 0x27EF, 66},
{"Supplemental Arrows-A"                    , 0x27F0  , 0x27FF, 67},
{"Braille"                                  , 0x2800  , 0x28FF, 68},
{"Supplemental Arrows-B"                    , 0x2900  , 0x297F, 70},
{"Miscellaneous Mathematical Symbols-B"     , 0x2980  , 0x29FF, 71},
{"Supplemental Mathematical Operators"      , 0x2A00  , 0x2AFF, 72},
{"(0x2B00 - 0x2E7F)"                        , 0x2B00  , 0x2E7F, 73},
{"CJK Radicals Supplement"                  , 0x2E80  , 0x2EFF, 74},
{"Kangxi Radicals"                          , 0x2F00  , 0x2FDF, 75},
{"(0x2FE0 - 0x2FEF)"                        , 0x2FE0  , 0x2FEF, 76},
{"Ideographic Description"                  , 0x2FF0  , 0x2FFF, 77},
{"CJK Symbols and Punctuation"              , 0x3000  , 0x303F, 78},
{"Hiragana"                                 , 0x3040  , 0x309F, 79},
{"Katakana"                                 , 0x30A0  , 0x30FF, 80},
{"Bopomofo"                                 , 0x3100  , 0x312F, 81},
{"Hangul Compatibility Jamo"                , 0x3130  , 0x318F, 82},
{"Kanbun"                                   , 0x3190  , 0x319F, 83},
{"Extended Bopomofo"                        , 0x31A0  , 0x31BF, 84},
{"(0x31C0 - 0x31EF)"                        , 0x31C0  , 0x31EF, 85},
{"Katakana Phonetic Extensions"             , 0x31F0  , 0x31FF, 86},
{"Enclosed CJK Letters and Months"          , 0x3200  , 0x32FF, 87},
{"CJK Compatibility"                        , 0x3300  , 0x33FF, 88},
{"CJK Unified Ideographs Extension A (1/26)"       , 0x3400  , 0x34FF, 89},
{"CJK Unified Ideographs Extension A (2/26)"       , 0x3500  , 0x35FF, 89},
{"CJK Unified Ideographs Extension A (3/26)"       , 0x3600  , 0x36FF, 89},
{"CJK Unified Ideographs Extension A (4/26)"       , 0x3700  , 0x37FF, 89},
{"CJK Unified Ideographs Extension A (5/26)"       , 0x3800  , 0x38FF, 89},
{"CJK Unified Ideographs Extension A (6/26)"       , 0x3900  , 0x39FF, 89},
{"CJK Unified Ideographs Extension A (7/26)"       , 0x3A00  , 0x3AFF, 89},
{"CJK Unified Ideographs Extension A (8/26)"       , 0x3B00  , 0x3BFF, 89},
{"CJK Unified Ideographs Extension A (9/26)"       , 0x3C00  , 0x3CFF, 89},
{"CJK Unified Ideographs Extension A (10/26)"       , 0x3D00  , 0x3DFF, 89},
{"CJK Unified Ideographs Extension A (11/26)"       , 0x3E00  , 0x3EFF, 89},
{"CJK Unified Ideographs Extension A (12/26)"       , 0x3F00  , 0x3FFF, 89},
{"CJK Unified Ideographs Extension A (13/26)"       , 0x4000  , 0x40FF, 89},
{"CJK Unified Ideographs Extension A (14/26)"       , 0x4100  , 0x41FF, 89},
{"CJK Unified Ideographs Extension A (15/26)"       , 0x4200  , 0x42FF, 89},
{"CJK Unified Ideographs Extension A (16/26)"       , 0x4300  , 0x43FF, 89},
{"CJK Unified Ideographs Extension A (17/26)"       , 0x4400  , 0x44FF, 89},
{"CJK Unified Ideographs Extension A (18/26)"       , 0x4500  , 0x45FF, 89},
{"CJK Unified Ideographs Extension A (19/26)"       , 0x4600  , 0x46FF, 89},
{"CJK Unified Ideographs Extension A (20/26)"       , 0x4700  , 0x47FF, 89},
{"CJK Unified Ideographs Extension A (21/26)"       , 0x4800  , 0x48FF, 89},
{"CJK Unified Ideographs Extension A (22/26)"       , 0x4900  , 0x49FF, 89},
{"CJK Unified Ideographs Extension A (23/26)"       , 0x4A00  , 0x4AFF, 89},
{"CJK Unified Ideographs Extension A (24/26)"       , 0x4B00  , 0x4BFF, 89},
{"CJK Unified Ideographs Extension A (25/26)"       , 0x4C00  , 0x4CFF, 89},
{"CJK Unified Ideographs Extension A (26/26)"       , 0x4D00  , 0x4DBF, 89},
{"(0x4DC0 - 0x4DFF)"                        , 0x4DC0  , 0x4DFF, 90},
{"CJK Unified Ideographs (1/82)"                   , 0x4E00  , 0x4EFF, 91},
{"CJK Unified Ideographs (2/82)"                   , 0x4F00  , 0x4FFF, 91},
{"CJK Unified Ideographs (3/82)"                   , 0x5000  , 0x50FF, 91},
{"CJK Unified Ideographs (4/82)"                   , 0x5100  , 0x51FF, 91},
{"CJK Unified Ideographs (5/82)"                   , 0x5200  , 0x52FF, 91},
{"CJK Unified Ideographs (6/82)"                   , 0x5300  , 0x53FF, 91},
{"CJK Unified Ideographs (7/82)"                   , 0x5400  , 0x54FF, 91},
{"CJK Unified Ideographs (8/82)"                   , 0x5500  , 0x55FF, 91},
{"CJK Unified Ideographs (9/82)"                   , 0x5600  , 0x56FF, 91},
{"CJK Unified Ideographs (10/82)"                   , 0x5700  , 0x57FF, 91},
{"CJK Unified Ideographs (11/82)"                   , 0x5800  , 0x58FF, 91},
{"CJK Unified Ideographs (12/82)"                   , 0x5900  , 0x59FF, 91},
{"CJK Unified Ideographs (13/82)"                   , 0x5A00  , 0x5AFF, 91},
{"CJK Unified Ideographs (14/82)"                   , 0x5B00  , 0x5BFF, 91},
{"CJK Unified Ideographs (15/82)"                   , 0x5C00  , 0x5CFF, 91},
{"CJK Unified Ideographs (16/82)"                   , 0x5D00  , 0x5DFF, 91},
{"CJK Unified Ideographs (17/82)"                   , 0x5E00  , 0x5EFF, 91},
{"CJK Unified Ideographs (18/82)"                   , 0x5F00  , 0x5FFF, 91},
{"CJK Unified Ideographs (19/82)"                   , 0x6000  , 0x60FF, 91},
{"CJK Unified Ideographs (20/82)"                   , 0x6100  , 0x61FF, 91},
{"CJK Unified Ideographs (21/82)"                   , 0x6200  , 0x62FF, 91},
{"CJK Unified Ideographs (22/82)"                   , 0x6300  , 0x63FF, 91},
{"CJK Unified Ideographs (23/82)"                   , 0x6400  , 0x64FF, 91},
{"CJK Unified Ideographs (24/82)"                   , 0x6500  , 0x65FF, 91},
{"CJK Unified Ideographs (25/82)"                   , 0x6600  , 0x66FF, 91},
{"CJK Unified Ideographs (26/82)"                   , 0x6700  , 0x67FF, 91},
{"CJK Unified Ideographs (27/82)"                   , 0x6800  , 0x68FF, 91},
{"CJK Unified Ideographs (28/82)"                   , 0x6900  , 0x69FF, 91},
{"CJK Unified Ideographs (29/82)"                   , 0x6A00  , 0x6AFF, 91},
{"CJK Unified Ideographs (30/82)"                   , 0x6B00  , 0x6BFF, 91},
{"CJK Unified Ideographs (31/82)"                   , 0x6C00  , 0x6CFF, 91},
{"CJK Unified Ideographs (32/82)"                   , 0x6D00  , 0x6DFF, 91},
{"CJK Unified Ideographs (33/82)"                   , 0x6E00  , 0x6EFF, 91},
{"CJK Unified Ideographs (34/82)"                   , 0x6F00  , 0x6FFF, 91},
{"CJK Unified Ideographs (35/82)"                   , 0x7000  , 0x70FF, 91},
{"CJK Unified Ideographs (36/82)"                   , 0x7100  , 0x71FF, 91},
{"CJK Unified Ideographs (37/82)"                   , 0x7200  , 0x72FF, 91},
{"CJK Unified Ideographs (38/82)"                   , 0x7300  , 0x73FF, 91},
{"CJK Unified Ideographs (39/82)"                   , 0x7400  , 0x74FF, 91},
{"CJK Unified Ideographs (40/82)"                   , 0x7500  , 0x75FF, 91},
{"CJK Unified Ideographs (41/82)"                   , 0x7600  , 0x76FF, 91},
{"CJK Unified Ideographs (42/82)"                   , 0x7700  , 0x77FF, 91},
{"CJK Unified Ideographs (43/82)"                   , 0x7800  , 0x78FF, 91},
{"CJK Unified Ideographs (44/82)"                   , 0x7900  , 0x79FF, 91},
{"CJK Unified Ideographs (45/82)"                   , 0x7A00  , 0x7AFF, 91},
{"CJK Unified Ideographs (46/82)"                   , 0x7B00  , 0x7BFF, 91},
{"CJK Unified Ideographs (47/82)"                   , 0x7C00  , 0x7CFF, 91},
{"CJK Unified Ideographs (48/82)"                   , 0x7D00  , 0x7DFF, 91},
{"CJK Unified Ideographs (49/82)"                   , 0x7E00  , 0x7EFF, 91},
{"CJK Unified Ideographs (50/82)"                   , 0x7F00  , 0x7FFF, 91},
{"CJK Unified Ideographs (51/82)"                   , 0x8000  , 0x80FF, 91},
{"CJK Unified Ideographs (52/82)"                   , 0x8100  , 0x81FF, 91},
{"CJK Unified Ideographs (53/82)"                   , 0x8200  , 0x82FF, 91},
{"CJK Unified Ideographs (54/82)"                   , 0x8300  , 0x83FF, 91},
{"CJK Unified Ideographs (55/82)"                   , 0x8400  , 0x84FF, 91},
{"CJK Unified Ideographs (56/82)"                   , 0x8500  , 0x85FF, 91},
{"CJK Unified Ideographs (57/82)"                   , 0x8600  , 0x86FF, 91},
{"CJK Unified Ideographs (58/82)"                   , 0x8700  , 0x87FF, 91},
{"CJK Unified Ideographs (59/82)"                   , 0x8800  , 0x88FF, 91},
{"CJK Unified Ideographs (60/82)"                   , 0x8900  , 0x89FF, 91},
{"CJK Unified Ideographs (61/82)"                   , 0x8A00  , 0x8AFF, 91},
{"CJK Unified Ideographs (62/82)"                   , 0x8B00  , 0x8BFF, 91},
{"CJK Unified Ideographs (63/82)"                   , 0x8C00  , 0x8CFF, 91},
{"CJK Unified Ideographs (64/82)"                   , 0x8D00  , 0x8DFF, 91},
{"CJK Unified Ideographs (65/82)"                   , 0x8E00  , 0x8EFF, 91},
{"CJK Unified Ideographs (66/82)"                   , 0x8F00  , 0x8FFF, 91},
{"CJK Unified Ideographs (67/82)"                   , 0x9000  , 0x90FF, 91},
{"CJK Unified Ideographs (68/82)"                   , 0x9100  , 0x91FF, 91},
{"CJK Unified Ideographs (69/82)"                   , 0x9200  , 0x92FF, 91},
{"CJK Unified Ideographs (70/82)"                   , 0x9300  , 0x93FF, 91},
{"CJK Unified Ideographs (71/82)"                   , 0x9400  , 0x94FF, 91},
{"CJK Unified Ideographs (72/82)"                   , 0x9500  , 0x95FF, 91},
{"CJK Unified Ideographs (73/82)"                   , 0x9600  , 0x96FF, 91},
{"CJK Unified Ideographs (74/82)"                   , 0x9700  , 0x97FF, 91},
{"CJK Unified Ideographs (75/82)"                   , 0x9800  , 0x98FF, 91},
{"CJK Unified Ideographs (76/82)"                   , 0x9900  , 0x99FF, 91},
{"CJK Unified Ideographs (77/82)"                   , 0x9A00  , 0x9AFF, 91},
{"CJK Unified Ideographs (78/82)"                   , 0x9B00  , 0x9BFF, 91},
{"CJK Unified Ideographs (79/82)"                   , 0x9C00  , 0x9CFF, 91},
{"CJK Unified Ideographs (80/82)"                   , 0x9D00  , 0x9DFF, 91},
{"CJK Unified Ideographs (81/82)"                   , 0x9E00  , 0x9EFF, 91},
{"CJK Unified Ideographs (82/82)"                   , 0x9F00  , 0x9FFF, 91},
{"Yi (1/5)"                                       , 0xA000  , 0xA0FF, 92},
{"Yi (2/5)"                                       , 0xA100  , 0xA1FF, 92},
{"Yi (3/5)"                                       , 0xA200  , 0xA2FF, 92},
{"Yi (4/5)"                                       , 0xA300  , 0xA3FF, 92},
{"Yi (5/5)"                                       , 0xA400  , 0xA48F, 92},
{"Yi Radicals"                              , 0xA480  , 0xA4CF, 93},
{"(0xA4D0 - 0xABFF)"                        , 0xA4D0  , 0xABFF, 94},
{"Hangul (1/44)"                                   , 0xAC00  , 0xACFF, 95},
{"Hangul (2/44)"                                   , 0xAD00  , 0xADFF, 95},
{"Hangul (3/44)"                                   , 0xAE00  , 0xAEFF, 95},
{"Hangul (4/44)"                                   , 0xAF00  , 0xAFFF, 95},
{"Hangul (5/44)"                                   , 0xB000  , 0xB0FF, 95},
{"Hangul (6/44)"                                   , 0xB100  , 0xB1FF, 95},
{"Hangul (7/44)"                                   , 0xB200  , 0xB2FF, 95},
{"Hangul (8/44)"                                   , 0xB300  , 0xB3FF, 95},
{"Hangul (9/44)"                                   , 0xB400  , 0xB4FF, 95},
{"Hangul (10/44)"                                   , 0xB500  , 0xB5FF, 95},
{"Hangul (11/44)"                                   , 0xB600  , 0xB6FF, 95},
{"Hangul (12/44)"                                   , 0xB700  , 0xB7FF, 95},
{"Hangul (13/44)"                                   , 0xB800  , 0xB8FF, 95},
{"Hangul (14/44)"                                   , 0xB900  , 0xB9FF, 95},
{"Hangul (15/44)"                                   , 0xBA00  , 0xBAFF, 95},
{"Hangul (16/44)"                                   , 0xBB00  , 0xBBFF, 95},
{"Hangul (17/44)"                                   , 0xBC00  , 0xBCFF, 95},
{"Hangul (18/44)"                                   , 0xBD00  , 0xBDFF, 95},
{"Hangul (19/44)"                                   , 0xBE00  , 0xBEFF, 95},
{"Hangul (20/44)"                                   , 0xBF00  , 0xBFFF, 95},
{"Hangul (21/44)"                                   , 0xC000  , 0xC0FF, 95},
{"Hangul (22/44)"                                   , 0xC100  , 0xC1FF, 95},
{"Hangul (23/44)"                                   , 0xC200  , 0xC2FF, 95},
{"Hangul (24/44)"                                   , 0xC300  , 0xC3FF, 95},
{"Hangul (25/44)"                                   , 0xC400  , 0xC4FF, 95},
{"Hangul (26/44)"                                   , 0xC500  , 0xC5FF, 95},
{"Hangul (27/44)"                                   , 0xC600  , 0xC6FF, 95},
{"Hangul (28/44)"                                   , 0xC700  , 0xC7FF, 95},
{"Hangul (29/44)"                                   , 0xC800  , 0xC8FF, 95},
{"Hangul (30/44)"                                   , 0xC900  , 0xC9FF, 95},
{"Hangul (31/44)"                                   , 0xCA00  , 0xCAFF, 95},
{"Hangul (32/44)"                                   , 0xCB00  , 0xCBFF, 95},
{"Hangul (33/44)"                                   , 0xCC00  , 0xCCFF, 95},
{"Hangul (34/44)"                                   , 0xCD00  , 0xCDFF, 95},
{"Hangul (35/44)"                                   , 0xCE00  , 0xCEFF, 95},
{"Hangul (36/44)"                                   , 0xCF00  , 0xCFFF, 95},
{"Hangul (37/44)"                                   , 0xD000  , 0xD0FF, 95},
{"Hangul (38/44)"                                   , 0xD100  , 0xD1FF, 95},
{"Hangul (39/44)"                                   , 0xD200  , 0xD2FF, 95},
{"Hangul (40/44)"                                   , 0xD300  , 0xD3FF, 95},
{"Hangul (41/44)"                                   , 0xD400  , 0xD4FF, 95},
{"Hangul (42/44)"                                   , 0xD500  , 0xD5FF, 95},
{"Hangul (43/44)"                                   , 0xD600  , 0xD6FF, 95},
{"Hangul (44/44)"                                   , 0xD700  , 0xD7A3, 95},
{"(0xD7A4 - 0xD7FF)"                        , 0xD7A4  , 0xD7FF, 96},
{"Surrogates (1/8)"                               , 0xD800  , 0xD8FF, 97},
{"Surrogates (2/8)"                               , 0xD900  , 0xD9FF, 97},
{"Surrogates (3/8)"                               , 0xDA00  , 0xDAFF, 97},
{"Surrogates (4/8)"                               , 0xDB00  , 0xDBFF, 97},
{"Surrogates (5/8)"                               , 0xDC00  , 0xDCFF, 97},
{"Surrogates (6/8)"                               , 0xDD00  , 0xDDFF, 97},
{"Surrogates (7/8)"                               , 0xDE00  , 0xDEFF, 97},
{"Surrogates (8/8)"                               , 0xDF00  , 0xDFFF, 97},
{"Private Use Area (1/26)"                         , 0xE000  , 0xE0FF, 98},
{"Private Use Area (2/26)"                         , 0xE100  , 0xE1FF, 98},
{"Private Use Area (3/26)"                         , 0xE200  , 0xE2FF, 98},
{"Private Use Area (4/26)"                         , 0xE300  , 0xE3FF, 98},
{"Private Use Area (5/26)"                         , 0xE400  , 0xE4FF, 98},
{"Private Use Area (6/26)"                         , 0xE500  , 0xE5FF, 98},
{"Private Use Area (7/26)"                         , 0xE600  , 0xE6FF, 98},
{"Private Use Area (8/26)"                         , 0xE700  , 0xE7FF, 98},
{"Private Use Area (9/26)"                         , 0xE800  , 0xE8FF, 98},
{"Private Use Area (10/26)"                         , 0xE900  , 0xE9FF, 98},
{"Private Use Area (11/26)"                         , 0xEA00  , 0xEAFF, 98},
{"Private Use Area (12/26)"                         , 0xEB00  , 0xEBFF, 98},
{"Private Use Area (13/26)"                         , 0xEC00  , 0xECFF, 98},
{"Private Use Area (14/26)"                         , 0xED00  , 0xEDFF, 98},
{"Private Use Area (15/26)"                         , 0xEE00  , 0xEEFF, 98},
{"Private Use Area (16/26)"                         , 0xEF00  , 0xEFFF, 98},
{"Private Use Area (17/26)"                         , 0xF000  , 0xF0FF, 98},
{"Private Use Area (18/26)"                         , 0xF100  , 0xF1FF, 98},
{"Private Use Area (19/26)"                         , 0xF200  , 0xF2FF, 98},
{"Private Use Area (20/26)"                         , 0xF300  , 0xF3FF, 98},
{"Private Use Area (21/26)"                         , 0xF400  , 0xF4FF, 98},
{"Private Use Area (23/26)"                         , 0xF500  , 0xF5FF, 98},
{"Private Use Area (24/26)"                         , 0xF600  , 0xF6FF, 98},
{"Private Use Area (25/26)"                         , 0xF700  , 0xF7FF, 98},
{"Private Use Area (26/26)"                         , 0xF800  , 0xF8FF, 98},
{"CJK Compatibility Ideographs (1/2)"             , 0xF900  , 0xF9FF, 99},
{"CJK Compatibility Ideographs (2/2)"             , 0xFA00  , 0xFAFF, 99},
{"Alphabetical Presentation Forms"          , 0xFB00  , 0xFB4F, 100},
{"Arabic Presentation Forms-A (1/3)"              , 0xFB50  , 0xFBFF, 101},
{"Arabic Presentation Forms-A (2/3)"              , 0xFC00  , 0xFCFF, 101},
{"Arabic Presentation Forms-A (3/3)"              , 0xFD00  , 0xFDFF, 101},
{"(0xFE00 - 0xFE1F)"                        , 0xFE00  , 0xFE1F, 102},
{"Combining Half Marks"                     , 0xFE20  , 0xFE2F, 103},
{"CJK Compatibility Forms"                  , 0xFE30  , 0xFE4F, 104},
{"Small Form Variants"                      , 0xFE50  , 0xFE6F, 105}, 
{"Arabic Presentation Forms-B"              , 0xFE70  , 0xFEFF, 106},
{"Halfwidth and Fullwidth Forms"            , 0xFF00  , 0xFFEF, 107},
{"Specials"                                 , 0xFFF0  , 0xFFFF, 108},
/*
{"(0x10000 - 0x102FF)"                      , 0x10000 , 0x102FF},
{"Old Italic"                               , 0x10300 , 0x1032F},
{"Gothic"                                   , 0x10330 , 0x1034F}, 
{"(0x10350 - 0x1043F)"                      , 0x10350 , 0x1043F},
{"Deseret"                                  , 0x10440 , 0x1044F}, 
{"(0x10450 - 0x1CFFF)"                      , 0x10450 , 0x1CFFF},
{"Byzantine Musical Symbols"                , 0x1D000 , 0x1D0FF},
{"Musical Symbols"                          , 0x1D100 , 0x1D1FF},
{"(0x1D200 - 0x1D3FF)"                      , 0x1D200 , 0x1D3FF},
{"Mathematical Alphanumeric Symbols"        , 0x1D400 , 0x1D7FF},
{"(0x1D800 - 0x1FFFF)"                      , 0x1D800 , 0x1FFFF},
{"CJK Unified Ideographs Extension B"       , 0x20000 , 0x2A6DF},
{"(0x2A6E0 - 0x2F7FF)"                      , 0x2A6E0 , 0x2F7FF},
{"CJK Compatibility Ideographs Supplement"  , 0x2F800 , 0x2FA1F}, 
{"(0x2FA20 - 0xFFF7F)"                      , 0x2FA20 , 0xFFF7F},
{"Private Use (Plane 15)"                   , 0xFFF80 , 0xFFFFF}, 
{"(0x100000 - 0x10FF7F)"                    , 0x100000, 0x10FF7F},
{"Private Use (Plane 16)"                   , 0x10FF80, 0x10FFFF},
{"(0x110000 - 0xFFFFFF)"                    , 0x110000, 0xFFFFFF},
*/
};

const int numUnicodeSubsets = sizeof(UnicodeSubsets)/sizeof(UnicodeSubset_t);

// This doesn't work for all fonts, only those that have these properties set
// It's better to check the available glyphs directly with GetGlyphIndices
void ListAvailableUnicodeSubsets(const char *fontname)
{
	HDC dc = GetDC(0);

	HFONT font = CreateFont(10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, fontname);
	HFONT oldFont = (HFONT)SelectObject(dc, font);

	for( UINT n = 0; n < numUnicodeSubsets; n++ )
	{
		UnicodeSubsets[n].isAvailable = false;
	}

	DWORD size = fGetFontUnicodeRanges(dc, 0);
	if( size )
	{
		GLYPHSET *gs = (GLYPHSET*)new char[size];
		gs->cbThis = size;
		DWORD ret = fGetFontUnicodeRanges(dc, gs);
		if( ret )
		{
			for( UINT n = 0; n < gs->cRanges; n++ )
			{
				for( UINT c = 0; c < numUnicodeSubsets; c++ )
				{
					if( UnicodeSubsets[c].range[0] <= gs->ranges[n].wcLow &&
						UnicodeSubsets[c].range[1] >= gs->ranges[n].wcLow )
					{
						UnicodeSubsets[c].isAvailable = true;
	
						while( gs->ranges[n].wcLow + gs->ranges[n].cGlyphs > 
							   UnicodeSubsets[c].range[1] + 1 )
						{
							c++;
							UnicodeSubsets[c].isAvailable = true;
						}

						break;
					}
				}
			}
		}
		delete[] (char*)gs;
	}

	SelectObject(dc, oldFont);
	DeleteObject(font);
	ReleaseDC(0, dc);
}

// This doesn't work for all fonts, only those that have these properties set
// It's better to check the available glyphs directly with GetGlyphIndices
bool DoesFontHaveUnicode(const char *fontname)
{
	bool hasUnicode = false;

	HDC dc = GetDC(0);
	HFONT font = CreateFontA(10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, fontname);
	HFONT oldFont = (HFONT)SelectObject(dc, font);
	
	FONTSIGNATURE fs;
	memset(&fs, 0, sizeof(fs));
	int r = GetTextCharsetInfo(dc, &fs, 0);
	if( r != DEFAULT_CHARSET && // Success?
		fs.fsUsb[3] & 0x80000000 && // Has unicode info?
		(fs.fsUsb[0] || fs.fsUsb[1] || fs.fsUsb[2] || fs.fsUsb[3]) ) // Which unicode subsets?
		hasUnicode = true;
	else
	{
		CHARSETINFO csi;
		memset(&csi, 0, sizeof(csi));
		r = TranslateCharsetInfo((DWORD*)r, &csi, TCI_SRCCHARSET);
		if( r && // Success?
			csi.fs.fsUsb[3] & 0x80000000 && // Has unicode info?
			(csi.fs.fsUsb[0] || csi.fs.fsUsb[1] || csi.fs.fsUsb[2] || csi.fs.fsUsb[3]) ) // Which unicode subsets?
			hasUnicode = true;
	}

	SelectObject(dc, oldFont);
	DeleteObject(font);
	ReleaseDC(0, dc);

	hasUnicode = true;
	return hasUnicode;
}

string GetCharSetName(int charSet)
{
	string str;

	switch( charSet )
	{
	case ANSI_CHARSET:
		str = "ANSI";
		break;
	case DEFAULT_CHARSET:
		str = "DEFAULT";
		break;
	case SYMBOL_CHARSET:          
		str = "SYMBOL";
		break;
	case SHIFTJIS_CHARSET:        
		str = "SHIFTJIS";
		break;
	case HANGUL_CHARSET:          
		str = "HANGUL";
		break;
	case GB2312_CHARSET:          
		str = "GB2312";
		break;
	case CHINESEBIG5_CHARSET:     
		str = "CHINESEBIG5";
		break;
	case OEM_CHARSET:             
		str = "OEM";
		break;
	case 130: // JOHAB_CHARSET           
		str = "JOHAB";
		break;
	case 177: // HEBREW_CHARSET          
		str = "HEBREW";
		break;
	case 178: // ARABIC_CHARSET          
		str = "ARABIC";
		break;
	case 161: // GREEK_CHARSET           
		str = "GREEK";
		break;
	case 162: // TURKISH_CHARSET         
		str = "TURKISH";
		break;
	case 163: // VIETNAMESE_CHARSET      
		str = "VIETNAMESE";
		break;
	case 222: // THAI_CHARSET            
		str = "THAI";
		break;
	case 238: // EASTEUROPE_CHARSET      
		str = "EASTEUROPE";
		break;
	case 204: // RUSSIAN_CHARSET         
		str = "RUSSIAN";
		break;
	case 77:  // MAC_CHARSET             
		str = "MAC";
		break;
	case 186: // BALTIC_CHARSET          
		str = "BALTIC";
		break;

	default:
		str = acStringFormat("%d", charSet);
	}
	
	return str;
}

int GetCharSet(const char *charSetName)
{
	string str = charSetName;
	int set = 0;

	if( str == "ANSI" )             set = 0;
	else if( str == "DEFAULT" )     set = 1;
	else if( str == "SYMBOL" )      set = 2;
	else if( str == "SHIFTJIS" )    set = 128;
	else if( str == "HANGUL" )      set = 129;
	else if( str == "GB2312" )      set = 134;
	else if( str == "CHINESEBIG5" ) set = 136;
	else if( str == "OEM" )         set = 255;
	else if( str == "JOHAB" )       set = 130;
	else if( str == "HEBREW" )      set = 177;
	else if( str == "ARABIC" )      set = 178;
	else if( str == "GREEK" )       set = 161;
	else if( str == "TURKISH" )     set = 162;
	else if( str == "VIETNAMESE" )  set = 163;
	else if( str == "THAI" )        set = 222;
	else if( str == "EASTEUROPE" )  set = 238;
	else if( str == "RUSSIAN" )     set = 204;
	else if( str == "MAC" )         set = 77;
	else if( str == "BALTIC" )      set = 186;
	else set = acStringScanInt(charSetName, 10, 0);

	return set;
}
